@@ -41,6 +41,7 @@
 #include "protos.h"
 
 extern struct topo_info info;
+extern struct settings settings;
 
 /* There are 5 map series,
  * level 1 is the state as a whole on the screen
@@ -107,7 +108,7 @@ struct section_dir {
 static struct section *temp_section_head;
 
 /* Prototypes ... */
-static int add_archive ( char * );
+static int add_new_archive ( char * );
 static int add_disk ( char *, char * );
 static void add_full_usa ( char *, char * );
 static int add_dir ( char *, char * );
@@ -163,7 +164,7 @@ add_file_method ( struct series *sp, char *path )
 	struct method *xp;
 	struct tpq_info *tp;
 
-	xp = (struct method *) malloc ( sizeof(struct method) );
+	xp = (struct method *) gmalloc ( sizeof(struct method) );
 	if ( ! xp )
 	    error ("file method - out of memory\n");
 
@@ -188,7 +189,7 @@ add_section_method ( struct series *sp, struct section *head )
 {
 	struct method *xp;
 
-	xp = (struct method *) malloc ( sizeof(struct method) );
+	xp = (struct method *) gmalloc ( sizeof(struct method) );
 	if ( ! xp )
 	    error ("section method - out of memory\n");
 
@@ -344,6 +345,7 @@ series_init ( void )
 	    /* Correct south of Tucson */
 	    sp->xdim = 435;
 	    sp->ydim = 256;
+	    sp->terra = 0;
 
 	    sp->lat_count = 10;
 	    sp->long_count = 5;
@@ -353,6 +355,8 @@ series_init ( void )
 	    sp->maplet_long_deg = 0.0250;
 	    sp->quad_lat_count = 1;
 	    sp->quad_long_count = 1;
+	    sp->x_pixel_scale = sp->maplet_long_deg / (double) sp->xdim;
+	    sp->y_pixel_scale = sp->maplet_lat_deg / (double) sp->ydim;
 
 	/* 2 of these in a square degree
 	 * one of top of the other a1 and e1
@@ -363,6 +367,7 @@ series_init ( void )
 	    /* Correct near Tucson */
 	    sp->xdim = 333;
 	    sp->ydim = 393;
+	    sp->terra = 0;
 
 	    sp->lat_count = 8;
 	    sp->long_count = 16;
@@ -372,6 +377,8 @@ series_init ( void )
 	    sp->maplet_long_deg = 0.0625;
 	    sp->quad_lat_count = 4;
 	    sp->quad_long_count = 8;
+	    sp->x_pixel_scale = sp->maplet_long_deg / (double) sp->xdim;
+	    sp->y_pixel_scale = sp->maplet_lat_deg / (double) sp->ydim;
 
 	/* This varies from state to state, and the only
 	 * thing that seems constant is that the maplets
@@ -383,6 +390,7 @@ series_init ( void )
 	    /* Correct in Southern Nevada */
 	    sp->xdim = 380;
 	    sp->ydim = 480;
+	    sp->terra = 0;
 
 	    /* The following is only correct for Nevada
 	     * for nevada these are g-files
@@ -395,6 +403,8 @@ series_init ( void )
 	    sp->maplet_long_deg = 0.5;
 	    sp->quad_lat_count = 8;
 	    sp->quad_long_count = 8;
+	    sp->x_pixel_scale = sp->maplet_long_deg / (double) sp->xdim;
+	    sp->y_pixel_scale = sp->maplet_lat_deg / (double) sp->ydim;
 
 	/* XXX */
 	sp = &info.series_info[S_ATLAS];
@@ -406,6 +416,7 @@ series_init ( void )
 
 	    sp->xdim = 290;
 	    sp->ydim = 364;
+	    sp->terra = 0;
 
 	    sp->lat_count = 1;
 	    sp->long_count = 1;
@@ -413,6 +424,8 @@ series_init ( void )
 	    sp->long_count_d = 1;
 	    sp->quad_lat_count = 1;
 	    sp->quad_long_count = 1;
+	    sp->x_pixel_scale = sp->maplet_long_deg / (double) sp->xdim;
+	    sp->y_pixel_scale = sp->maplet_lat_deg / (double) sp->ydim;
 
 	/* XXX - The entire state */
 	sp = &info.series_info[S_STATE];
@@ -426,6 +439,8 @@ series_init ( void )
 	    sp->quad_lat_count = 1;
 	    sp->quad_long_count = 1;
 
+	    sp->terra = 0;
+
 #ifdef notdef
 	    /* XXX - true for california */
 	    sp->maplet_lat_deg = 10.0;
@@ -456,17 +471,106 @@ series_init ( void )
 
 	    sp->xdim = 309;
 	    sp->ydim = 256;
+	    sp->x_pixel_scale = sp->maplet_long_deg / (double) sp->xdim;
+	    sp->y_pixel_scale = sp->maplet_lat_deg / (double) sp->ydim;
+
+#ifdef TERRA
+	/* Terraserver 2m (like 1:24k) */
+	sp = &info.series_info[S_TOPO_2M];
+	    series_init_one ( sp, S_TOPO_2M );
+
+	    sp->xdim = 200;
+	    sp->ydim = 200;
+	    sp->terra = 1;
+	    sp->x_pixel_scale = 2.0;
+	    sp->y_pixel_scale = 2.0;
+	    sp->scale_name = "Scale2m";
+
+	/* Terraserver 8m (like 1:100) */
+	sp = &info.series_info[S_TOPO_8M];
+	    series_init_one ( sp, S_TOPO_8M );
+
+	    sp->xdim = 200;
+	    sp->ydim = 200;
+	    sp->terra = 1;
+	    sp->x_pixel_scale = 8.0;
+	    sp->y_pixel_scale = 8.0;
+	    sp->scale_name = "Scale8m";
+
+	/* Terraserver 32m */
+	sp = &info.series_info[S_TOPO_32M];
+	    series_init_one ( sp, S_TOPO_32M );
+
+	    sp->xdim = 200;
+	    sp->ydim = 200;
+	    sp->terra = 1;
+	    sp->x_pixel_scale = 32.0;
+	    sp->y_pixel_scale = 32.0;
+	    sp->scale_name = "Scale32m";
+#endif
+}
+
+/* Added this list of archives 6-5-2008, to allow this
+ * to by dynamically handled via the config file.
+ * We now check for the existence of the directories
+ * before we add them to the list, so the list may be
+ * empty by the time we try to initialize (which is a
+ * good thing, since we can declare an error then).
+ *
+ * Notice also that this scheme allows gtopo to handle
+ * an archive that is split into any number of directories,
+ * which I find handy on a laptop with space on two different
+ * disk partitions.
+ */
+struct archive_entry {
+	struct archive_entry *next;
+	char *path;
+};
+
+static struct archive_entry *archive_head = NULL;
+
+void
+archive_clear ( void )
+{
+	archive_head = NULL;
+}
+
+void
+archive_add ( char *path )
+{
+	struct archive_entry *ap;
+	struct archive_entry *lp;
+
+	if ( ! is_directory ( path ) )
+	    return;
+
+	ap = (struct archive_entry *) gmalloc ( sizeof(struct archive_entry) );
+	ap->next = NULL;
+	ap->path = strhide ( path );
+
+	/* keep list in order (if that matters, doesn't hurt) */
+	if ( ! archive_head )
+	    archive_head = ap;
+	else {
+	    lp = archive_head;
+	    while ( lp->next )
+	    	lp = lp->next;
+	    lp->next = ap;
+	}
 }
 
 /* This is the usual initialization when we want to setup to
  * view a whole collection of potentially multiple states.
  */
 int
-archive_init ( char *archives[] )
+archive_init ( void )
 {
-	char **p;
+	struct archive_entry *ap;
 	int nar;
 
+	if ( ! archive_head )
+	    return 0;
+
 	series_init ();
 
 	nar = 0;
@@ -474,8 +578,8 @@ archive_init ( char *archives[] )
 	info.n_sections = 0;
 
 	/* Look for the SI_D01 thing first off */
-	for ( p=archives; *p; p++ ) {
-	    if ( add_usa ( *p, 1 ) )
+	for ( ap = archive_head; ap; ap = ap->next ) {
+	    if ( add_usa ( ap->path, 1 ) )
 		nar++;
 	}
 
@@ -483,8 +587,8 @@ archive_init ( char *archives[] )
 
 	temp_section_head = NULL;
 
-	for ( p=archives; *p; p++ ) {
-	    if ( add_archive ( *p ) )
+	for ( ap = archive_head; ap; ap = ap->next ) {
+	    if ( add_new_archive ( ap->path ) )
 		nar++;
 	}
 
@@ -538,9 +642,11 @@ show_statistics ( void )
 	}
 }
 
-int
+static int
 try_series ( int new_series )
 {
+	int rv = 0;
+
 	/* Give it a whirl, see if we can load a maplet
 	 * for this series at the center position.
 	 * Trying to load the maplet avoids changing to
@@ -553,9 +659,13 @@ try_series ( int new_series )
 	 * into the cache, and we will soon be fetching it
 	 * to display anyway.
 	 */
-	if ( load_maplet ( info.long_maplet, info.lat_maplet ) )
-	    return 1;
-	return 0;
+	if ( load_maplet ( info.maplet_x, info.maplet_y ) )
+	    rv = 1;
+
+	if ( settings.verbose & V_BASIC )
+	    printf ( "try series returns %d for %d\n", rv, new_series );
+
+	return rv;
 }
 
 /* Move to a less detailed series */
@@ -564,14 +674,20 @@ up_series ( void )
 {
 	int series = info.series->series;
 
+	if ( settings.verbose & V_BASIC )
+	    printf ( "up series called on series %d\n", series );
+
 	if ( series == 0 )
 	    return;
 
 	if ( try_series ( series - 1 ) ) {
 	    redraw_series ();
+	    if ( settings.verbose & V_BASIC )
+		printf ( "up series moved to series %d\n", series-1 );
 	    return;
 	}
 
+	/* cannot switch, restore original series */
 	info.series = &info.series_info[series];
 	synch_position ();
 }
@@ -582,51 +698,33 @@ down_series ( void )
 {
 	int series = info.series->series;
 
+	if ( settings.verbose & V_BASIC )
+	    printf ( "down series called on series %d\n", series );
+
 	if ( series == N_SERIES - 1 )
 	    return;
 
 	if ( try_series ( series + 1 ) ) {
 	    redraw_series ();
+	    if ( settings.verbose & V_BASIC )
+		printf ( "down series moved to series %d\n", series+1 );
 	    return;
 	}
 
+	/* cannot switch, restore original series */
 	info.series = &info.series_info[series];
 	synch_position ();
 }
 
-#ifdef notdef
-/* no longer used, up/down series make more sense */
+/* Like set_series, but doesn't try to synch position */
 void
-toggle_series ( void )
+initial_series ( enum s_type s )
 {
-	int series;
-	int new_series;
+    	if ( s < 0 || s >= N_SERIES )
+	    error ( "initial_series, impossible value: %d\n", s );
 
-	/* remember where we started */
-	series = info.series->series;
-
-	if ( info.verbose & V_BASIC )
-	    printf ( "toggle from series %d (%s)\n", series+1, wonk_series ( series ) );
-
-	new_series = series;
-	do {
-	    if ( new_series == S_24K )
-	    	new_series = S_STATE;
-	    else
-		++new_series;
-
-	    if ( try_series ( new_series ) )
-	    	return;
-
-	} while ( new_series != series );
-
-	/* fall out the end when no other series has
-	 * methods (always happens in -f mode )
-	 */
-	info.series = &info.series_info[series];
-	synch_position ();
+	info.series = &info.series_info[s];
 }
-#endif
 
 void
 set_series ( enum s_type s )
@@ -637,7 +735,7 @@ set_series ( enum s_type s )
 	info.series = &info.series_info[s];
 
 	synch_position ();
-	if ( info.verbose & V_BASIC ) {
+	if ( settings.verbose & V_BASIC ) {
 	    printf ( "Switch to series %d (%s)\n", s+1, wonk_series ( s ) );
 	    show_methods ( info.series );
 	}
@@ -686,7 +784,7 @@ section_map_path ( struct section_dir *dp, int lat_section, int long_section, in
 
 	/* 1 - try all lower case */
 	sprintf ( path_buf, "%s/%c%2d%03d%c%c.tpq", dp->path, series_letter, lat_section, long_section, lat_q, long_q );
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "Trying %d %d -- %s\n", lat_quad, long_quad, path_buf );
 
 	if ( is_file(path_buf) )
@@ -697,7 +795,7 @@ section_map_path ( struct section_dir *dp, int lat_section, int long_section, in
 	series_letter = toupper(series_letter);
 
 	sprintf ( path_buf, "%s/%c%2d%03d%c%c.TPQ", dp->path, series_letter, lat_section, long_section, lat_q, long_q );
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "Trying %d %d -- %s\n", lat_quad, long_quad, path_buf );
 
 	if ( is_file(path_buf) )
@@ -705,7 +803,7 @@ section_map_path ( struct section_dir *dp, int lat_section, int long_section, in
 
 	/* 3 - try upper case name, with lower case .tpq */
 	sprintf ( path_buf, "%s/%c%2d%03d%c%c.tpq", dp->path, series_letter, lat_section, long_section, lat_q, long_q );
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "Trying %d %d -- %s\n", lat_quad, long_quad, path_buf );
 
 	if ( is_file(path_buf) )
@@ -716,7 +814,7 @@ section_map_path ( struct section_dir *dp, int lat_section, int long_section, in
 	series_letter = tolower(series_letter);
 
 	sprintf ( path_buf, "%s/%c%2d%03d%c%c.TPQ", dp->path, series_letter, lat_section, long_section, lat_q, long_q );
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "Trying %d %d -- %s\n", lat_quad, long_quad, path_buf );
 
 	if ( is_file(path_buf) )
@@ -761,31 +859,31 @@ section_find_map ( struct section *head, int lat_section, int long_section, int
  */
 static int
 method_file ( struct maplet *mp, struct method *xp,
-		int maplet_long, int maplet_lat )
+		int maplet_x, int maplet_y )
 {
 	int x_index, y_index;
 
 	/* Now figure which maplet within the sheet we need.
 	 */
-	mp->sheet_index_long = maplet_long - xp->tpq->sheet_long;
-	mp->sheet_index_lat = maplet_lat - xp->tpq->sheet_lat;
+	mp->sheet_x = maplet_x - xp->tpq->sheet_long;
+	mp->sheet_y = maplet_y - xp->tpq->sheet_lat;
 
-	if ( info.verbose & V_ARCHIVE ) {
+	if ( settings.verbose & V_ARCHIVE ) {
 	    printf ( "MF sheet long, lat: %d %d\n", xp->tpq->sheet_long, xp->tpq->sheet_lat );
-	    printf ( "MF point : %d %d\n", maplet_long, maplet_lat );
-	    printf ( "MF index: %d %d\n", mp->sheet_index_long, mp->sheet_index_lat );
+	    printf ( "MF maplet indices (x/y) : %d %d\n", maplet_x, maplet_y );
+	    printf ( "MF sheet indices (x/y): %d %d\n", mp->sheet_x, mp->sheet_y );
 	}
 
-	if ( mp->sheet_index_long < 0 || mp->sheet_index_long >= xp->tpq->long_count )
+	if ( mp->sheet_x < 0 || mp->sheet_x >= xp->tpq->long_count )
 	    return 0;
-	if ( mp->sheet_index_lat < 0 || mp->sheet_index_lat >= xp->tpq->lat_count )
+	if ( mp->sheet_y < 0 || mp->sheet_y >= xp->tpq->lat_count )
 	    return 0;
 
 	mp->tpq_path = xp->tpq->path;
 
 	/* flip the count to origin from the NW corner */
-	x_index = xp->tpq->long_count - mp->sheet_index_long - 1;
-	y_index = xp->tpq->lat_count - mp->sheet_index_lat - 1;
+	x_index = xp->tpq->long_count - mp->sheet_x - 1;
+	y_index = xp->tpq->lat_count - mp->sheet_y - 1;
 
 	mp->tpq_index = y_index * xp->tpq->long_count + x_index;
 
@@ -794,7 +892,7 @@ method_file ( struct maplet *mp, struct method *xp,
 
 static int
 method_section ( struct maplet *mp, struct method *xp,
-		    int maplet_long, int maplet_lat )
+		    int maplet_x, int maplet_y )
 {
 	struct series *sp;
 	int lat_section, long_section;
@@ -803,14 +901,14 @@ method_section ( struct maplet *mp, struct method *xp,
 
 	sp = info.series;
 
-	lat_section = maplet_lat / (sp->lat_count_d * sp->lat_count);
-	long_section = maplet_long / (sp->long_count_d * sp->long_count);
+	lat_section = maplet_y / (sp->lat_count_d * sp->lat_count);
+	long_section = maplet_x / (sp->long_count_d * sp->long_count);
 
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "lookup_quad, section: %d %d\n", lat_section, long_section );
 
-	lat_quad = maplet_lat / sp->lat_count - lat_section * sp->lat_count_d;
-	long_quad = maplet_long / sp->long_count - long_section * sp->long_count_d;
+	lat_quad = maplet_y / sp->lat_count - lat_section * sp->lat_count_d;
+	long_quad = maplet_x / sp->long_count - long_section * sp->long_count_d;
 
 	/* See if the map sheet is available.
 	 */
@@ -820,15 +918,15 @@ method_section ( struct maplet *mp, struct method *xp,
 
 	/* Now figure which maplet within the sheet we need.
 	 */
-	mp->sheet_index_long = maplet_long - long_quad * sp->long_count - long_section * sp->long_count * sp->long_count_d;
-	mp->sheet_index_lat = maplet_lat - lat_quad * sp->lat_count - lat_section * sp->lat_count * sp->lat_count_d;
+	mp->sheet_x = maplet_x - long_quad * sp->long_count - long_section * sp->long_count * sp->long_count_d;
+	mp->sheet_y = maplet_y - lat_quad * sp->lat_count - lat_section * sp->lat_count * sp->lat_count_d;
 
-	if ( info.verbose & V_ARCHIVE )
-	    printf ( "lat/long quad, lat/long maplet: %d %d  %d %d\n", lat_quad, long_quad, maplet_lat, maplet_long );
+	if ( settings.verbose & V_ARCHIVE )
+	    printf ( "long/lag quad, x/y maplet: %d %d  %d %d\n", long_quad, lat_quad, maplet_x, maplet_y );
 
 	/* flip the count to origin from the NW corner */
-	x_index = sp->long_count - mp->sheet_index_long - 1;
-	y_index = sp->lat_count - mp->sheet_index_lat - 1;
+	x_index = sp->long_count - mp->sheet_x - 1;
+	y_index = sp->lat_count - mp->sheet_y - 1;
 
 	mp->tpq_index = y_index * sp->long_count + x_index;
 
@@ -840,11 +938,11 @@ method_section ( struct maplet *mp, struct method *xp,
  * It returns with the following set:
  *	tpq_path
  *	tpq_index
- *	sheet_index_long
- *	sheet_index_lat
+ *	sheet_x
+ *	sheet_y
  */
 int
-lookup_series ( struct maplet *mp, int maplet_long, int maplet_lat )
+lookup_series ( struct maplet *mp, int maplet_x, int maplet_y )
 {
 	struct series *sp;
 	struct method *xp;
@@ -856,9 +954,9 @@ lookup_series ( struct maplet *mp, int maplet_long, int maplet_lat )
 	/* We skip STATE_METHOD in this loop */
 	for ( xp = sp->methods; xp; xp = xp->next ) {
 	    if ( xp->type == M_SECTION )
-	    	done = method_section ( mp, xp, maplet_long, maplet_lat );
+	    	done = method_section ( mp, xp, maplet_x, maplet_y );
 	    if ( xp->type == M_FILE )
-	    	done = method_file ( mp, xp, maplet_long, maplet_lat );
+	    	done = method_file ( mp, xp, maplet_x, maplet_y );
 	    if ( done )
 	    	return 1;
 	}
@@ -913,7 +1011,7 @@ add_usa ( char *archive, int depth )
 }
 
 static int
-add_archive ( char *archive )
+add_new_archive ( char *archive )
 {
 	DIR *dd;
 	struct dirent *dp;
@@ -962,7 +1060,7 @@ add_disk ( char *archive, char *disk )
 	if ( ! (dd = opendir ( disk_path )) )
 	    return 0;
 
-	if ( info.verbose & V_BASIC )
+	if ( settings.verbose & V_BASIC )
 	    printf ( "Found disk: %s\n", disk_path );
 
 	/* Loop through this disk image.
@@ -1110,7 +1208,7 @@ add_section ( char *disk, char *section, struct section **head )
 
 	sprintf ( section_path, "%s/%s", disk, section );
 
-	dp = (struct section_dir *) malloc ( sizeof(struct section_dir) );
+	dp = (struct section_dir *) gmalloc ( sizeof(struct section_dir) );
 	if ( ! dp )
 	    error ("Section new (dir) - out of memory\n");
 
@@ -1130,7 +1228,7 @@ add_section ( char *disk, char *section, struct section **head )
 
 	if ( ! ep ) {
 	    /* Does not yet exist on main list */
-	    ep = (struct section *) malloc ( sizeof(struct section) );
+	    ep = (struct section *) gmalloc ( sizeof(struct section) );
 	    if ( ! ep )
 		error ("Section new - out of memory\n");
 
@@ -1138,18 +1236,18 @@ add_section ( char *disk, char *section, struct section **head )
 	    ep->dir_head = (struct section_dir *) NULL;
 	    ep->dir_count = 0;
 
-	    if ( info.verbose & V_ARCHIVE )
+	    if ( settings.verbose & V_ARCHIVE )
 		printf ( "Added section:" );
 
 	    ep->next = *head;
 	    *head = ep;
 	    info.n_sections++;
 	} else {
-	    if ( info.verbose & V_ARCHIVE )
+	    if ( settings.verbose & V_ARCHIVE )
 		printf ( "Added section (border):" );
 	}
 
-	if ( info.verbose & V_ARCHIVE ) {
+	if ( settings.verbose & V_ARCHIVE ) {
 	    printf ( " %d  %s", latlong, dp->path );
 	    printf ( " %c %c %c\n",
 		    dp->tpq_code[S_500K],
@@ -1203,7 +1301,7 @@ dir_lookup ( char *name )
 	    }
 	}
 
-	dp = (struct dir_table *) malloc ( sizeof(struct dir_table) );
+	dp = (struct dir_table *) gmalloc ( sizeof(struct dir_table) );
 	if ( ! dp )
 	    error ("dir_lookup - out of memory\n");
 
@@ -1219,7 +1317,7 @@ add_dir_series ( int series, char *dirpath, char *name )
 	char tpq_path[100];
 
 	sprintf ( tpq_path, "%s/%s", dirpath, name );
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "add dir series:  %s\n", tpq_path );
 
 	if ( is_directory ( tpq_path ) )
@@ -1229,7 +1327,7 @@ add_dir_series ( int series, char *dirpath, char *name )
 	if ( info.have_usa )
 		return;
 
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "Add file/state method: %d  %s\n", series, tpq_path );
 
 	(void) add_file_method ( &info.series_info[series], tpq_path );
@@ -1268,7 +1366,7 @@ add_dir ( char *archive, char *dir )
 	if ( dir_lookup ( dir ) )
 	    return 0;
 
-	if ( info.verbose & V_ARCHIVE )
+	if ( settings.verbose & V_ARCHIVE )
 	    printf ( "add dir: %d %s\n", series, dir_path );
 
 	if ( ! (dd = opendir ( dir_path )) )
@@ -1414,7 +1512,7 @@ add_full_usa ( char *path, char *name )
 	if ( ! is_directory ( si_path ) )
 	    return;
 
-	if ( info.verbose & V_BASIC )
+	if ( settings.verbose & V_BASIC )
 	    printf ( "Found level 123 for full USA at %s\n", si_path );
 
 #ifdef notdef
