@@ -28,8 +28,12 @@
 #include "gtopo.h"
 #include "protos.h"
 
-#define MINIMUM_VIEW	100
-#define INITIAL_VIEW	800
+/* 640 x 800 works pretty well for printing onto
+ * 8.5 x 11 inch paper.
+ * Actually 640 x 828 has the same aspect ratio.
+ */
+#define INITIAL_VIEW_X	640
+#define INITIAL_VIEW_Y	800
 
 /* I have tried to group some "commonly fiddled"
  * parameters here for your convenience.  Someday
@@ -46,34 +50,6 @@
     S_24K;
 */
 
-#define INITIAL_SERIES	S_STATE
-
-/*
-#define INITIAL_ARCHIVE	"/u1/backroads"
-*/
-
-#ifdef notdef
-/* Mt. Hopkins, Arizona */
-#define INITIAL_LONG	-110.88
-#define INITIAL_LAT	31.69
-
-/* In California west of Taboose Pass */
-#define INITIAL_LONG	-dms2deg ( 118, 31, 0 )
-#define INITIAL_LAT	dms2deg ( 37, 1, 0 )
-
-/* Near Alturas, NE California */
-#define INITIAL_LONG	-120.5
-#define INITIAL_LAT	41.5
-
-/* Near Las Vegas, Nevada */
-#define INITIAL_LONG	-114.9894
-#define INITIAL_LAT	36.2338
-#endif
-
-/* Flagstaff, Arizona */
-#define INITIAL_LONG	-111.6722
-#define INITIAL_LAT	35.18
-
 /* gtopo.c - the main file for gtopo
  *
  * Tom Trebisky  MMT Observatory, Tucson, Arizona
@@ -148,6 +124,7 @@
  */
 
 struct topo_info info;
+struct settings settings;
 
 struct series series_info_buf[N_SERIES];
 
@@ -160,31 +137,47 @@ struct series series_info_buf[N_SERIES];
  * machines that keep the topos in different places.
  */
 
-#ifdef INITIAL_ARCHIVE
-char *topo_archives[] = { INITIAL_ARCHIVE, NULL };
-#else
-char *topo_archives[] = { "/u1/topo", "/u2/topo", "/mmt/topo", NULL };
-#endif
-
 GdkColormap *syscm;
 
+struct mouse {
+	double x;
+	double y;
+	int time;
+} mouse_info;
+
+/* XXX - should eliminate mo_ stuff below and use the above */
 struct viewport {
 	int vx;
 	int vy;
 	int vxcent;
 	int vycent;
+	double mo_x;
+	double mo_y;
+	int mo_time;
 	GtkWidget *da;
 } vp_info;
 
+enum info_status { GONE, HIDDEN, UP };
+
+struct info_info {
+	enum info_status status;
+	GtkWidget *main;
+	GtkWidget *l_long;
+	GtkWidget *l_lat;
+	GtkWidget *e_long;
+	GtkWidget *e_lat;
+} i_info = { GONE };
+
 /* Prototypes ..........
  */
 static void cursor_show ( int );
+static int try_position ( double, double );
 
 gint
 destroy_handler ( GtkWidget *w, GdkEvent *event, gpointer data )
 {
 	gtk_main_quit ();
-	if ( info.verbose & V_BASIC )
+	if ( settings.verbose & V_BASIC )
 	    show_statistics ();
 
 	return FALSE;
@@ -206,7 +199,7 @@ static int expose_count = 0;
 gint
 expose_handler ( GtkWidget *wp, GdkEventExpose *ep, gpointer data )
 {
-	if ( info.verbose & V_WINDOW && expose_count < 4 )
+	if ( settings.verbose & V_WINDOW && expose_count < 4 )
 	    printf ( "Expose event %d\n", expose_count++ );
 
     	pixmap_expose ( ep->area.x, ep->area.y, ep->area.width, ep->area.height );
@@ -243,7 +236,7 @@ state_handler ( struct maplet *mp )
 
 	tp = mp->tpq;
 
-	if ( info.verbose & V_BASIC ) {
+	if ( settings.verbose & V_BASIC ) {
 	    printf ( "State handler %s\n", mp->tpq->path );
 	    printf ( "Position, long, lat: %.4f %.4f\n", info.long_deg, info.lat_deg );
 	    printf ( "Sheet, S, N: %.4f %.4f\n", tp->s_lat, tp->n_lat );
@@ -283,7 +276,14 @@ state_handler ( struct maplet *mp )
 	draw_maplet ( mp, origx, origy );
 }
 
-/* This is the guts of what goes on during a reconfigure */
+/* This is the guts of what goes on during a reconfigure.
+ * Watch out for a multitude of sign conventions, here is a
+ * quick orientation:
+ * 	long,lat    - long increases to the east (right), lat increases to north (up)
+ * 	maplet x,y  - x increases to west (left), y increases to north (up)
+ * 	pixmap x,y  - x increases to the right (east), y increases down (south)
+ * 	utm x,y     - x increases to east, y increases to north.
+ */
 void
 pixmap_redraw ( void )
 {
@@ -326,7 +326,7 @@ pixmap_redraw ( void )
 	/* load the maplet containing the current position so
 	 * we can get the maplet pixel size up front.
 	 */
-	mp = load_maplet ( info.long_maplet, info.lat_maplet );
+	mp = load_maplet ( info.maplet_x, info.maplet_y );
 
 	/* The above can fail if we have used the mouse to wander off
 	 * the edge of the map coverage.  We use the series information
@@ -337,7 +337,7 @@ pixmap_redraw ( void )
 	if ( mp ) {
 	    info.series->xdim = mx = mp->xdim;
 	    info.series->ydim = my = mp->ydim;
-	    if ( info.verbose & V_DRAW )
+	    if ( settings.verbose & V_DRAW )
 		printf ( "Center maplet x,ydim = %d, %d\n", mx, my );
 	}
 
@@ -348,7 +348,7 @@ pixmap_redraw ( void )
 	origx = vp_info.vxcent - offx;
 	origy = vp_info.vycent - offy;
 
-	if ( info.verbose & V_DRAW )
+	if ( settings.verbose & V_DRAW )
 	    printf ( "Maplet off, orig: %d %d -- %d %d\n", offx, offy, origx, origy );
 
 	if ( info.center_only ) {
@@ -361,23 +361,32 @@ pixmap_redraw ( void )
 	    ny2 = + (origy + my - 1 ) / my;
 	}
 
-	if ( info.verbose & V_DRAW ) {
+	if ( settings.verbose & V_DRAW ) {
 	    printf ( "redraw -- viewport: %d %d -- maplet %d %d -- offset: %d %d\n",
 		vxdim, vydim, mx, my, offx, offy );
 	    printf ( "redraw range: x,y = %d %d %d %d\n", nx1, nx2, ny1, ny2 );
 	}
 
+	/* This loop works in maplet indices, with
+	 * x increasing to the west (left), and y increasing to the north (up).
+	 * which is exactly opposite of the GTK pixel coordinates, which have
+	 * the origin at the upper left of the screen.
+	 */
 	for ( y = ny1; y <= ny2; y++ ) {
 	    for ( x = nx1; x <= nx2; x++ ) {
 
-		mp = load_maplet ( info.long_maplet + x, info.lat_maplet + y );
+		if ( info.series->terra )
+		    mp = load_maplet ( info.maplet_x - x, info.maplet_y + y );
+		else
+		    mp = load_maplet ( info.maplet_x + x, info.maplet_y + y );
+
 		if ( ! mp ) {
-		    if ( info.verbose & V_DRAW2 )
+		    if ( settings.verbose & V_DRAW2 )
 			printf ( "redraw, no maplet at %d %d\n", x, y );
 		    continue;
 		}
 
-		if ( info.verbose & V_DRAW2 )
+		if ( settings.verbose & V_DRAW2 )
 		    printf ( "redraw OK for %d %d, draw at %d %d\n",
 			x, y, origx + mp->xdim*x, origy + mp->ydim*y );
 		draw_maplet ( mp,
@@ -386,7 +395,7 @@ pixmap_redraw ( void )
 	    }
 	}
 
-	if ( info.show_maplets ) {
+	if ( settings.show_maplets ) {
 	    for ( x = nx1+1; x <= nx2; x++ ) {
 		xx = origx - mx * x,
 		gdk_draw_line ( info.series->pixels, vp_info.da->style->black_gc,
@@ -418,7 +427,7 @@ configure_handler ( GtkWidget *wp, GdkEvent *event, gpointer data )
 	vp_info.vxcent = vp_info.vx / 2;
 	vp_info.vycent = vp_info.vy / 2;
 
-	if ( info.verbose & V_WINDOW )
+	if ( settings.verbose & V_WINDOW )
 	    printf ( "Configure event %d (%d, %d)\n", config_count++, vxdim, vydim );
 
 	for ( i=0; i<N_SERIES; i++ ) {
@@ -441,19 +450,40 @@ configure_handler ( GtkWidget *wp, GdkEvent *event, gpointer data )
  * them around and handling expose events very fast).
  * but to fiddle with pixel values, we need to copy them
  * to a client side Image or Pixbuf.
+ *
+ * With a 640 wide by 800 tall viewport, we get a pretty good
+ * fit to a single 8.5 x 11 sheet of paper.
+ * The "s" key spits out gtopo.jpg
+ * convert gtopo.jpg gtopo.ps ; lpr gtopo.ps
+ * This works -right now- and ain't bad at all.
+ *
+ * We save as a .jpg file, maybe we could have the p key do
+ * "print" and save a lossless format like png, or better yet
+ * direct to postscript, and even run convert ...
+ *
+ * Whether to force precise 7.5 minute quad scaling?
+ * This could be important if we were using these as base
+ * sheets for geologic mapping say, 1:24,000 is 1 inch = 2000 feet.
+ * We could apply a 4x factor and get 1 inch = 500 feet.
+ * However for best print copy, we want to avoid pixel interpolation
+ * and should just let the scale be whatever is needed for 1:1
+ * pixel mapping.
  */
 void
 snap ( void )
 {
 	GdkPixbuf *pixbuf;
 
-	printf ( "Snapshot\n" );
+	if ( settings.verbose & V_WINDOW ) {
+	    printf ( "Snapshot" );
 
-	if ( info.series->series != S_STATE ) {
-	    struct maplet *mp;
-	    mp = load_maplet ( info.long_maplet, info.lat_maplet );
-	    printf ( " from file: %s\n", mp->tpq->path );
-	    printf ( " quad: %s (%s)\n", mp->tpq->quad, mp->tpq->state );
+	    if ( info.series->series != S_STATE ) {
+		struct maplet *mp;
+		mp = load_maplet ( info.maplet_x, info.maplet_y );
+		printf ( " from file: %s", mp->tpq->path );
+		printf ( " quad: %s (%s)", mp->tpq->quad, mp->tpq->state );
+	    }
+	    printf ( "\n" );
 	}
 
 	pixbuf = gdk_pixbuf_get_from_drawable(NULL, info.series->pixels, NULL,
@@ -513,34 +543,10 @@ show_pos ( void )
 }
 
 void
-move_xy ( int new_x, int new_y )
+full_redraw ( void )
 {
-	int vxcent, vycent;
-	double dlat, dlong;
-	double x_pixel_scale, y_pixel_scale;
 	int i;
 
-	/* viewport center */
-	vxcent = vp_info.vx / 2;
-	vycent = vp_info.vy / 2;
-
-	if ( info.verbose & V_EVENT )
-	    printf ( "Button: orig position (lat/long) %.4f %.4f\n",
-		info.lat_deg, info.long_deg );
-
-	x_pixel_scale = info.series->maplet_long_deg / (double) info.series->xdim;
-	y_pixel_scale = info.series->maplet_lat_deg / (double) info.series->ydim;
-
-	dlat  = (new_y - (double)vycent) * y_pixel_scale;
-	dlong = (new_x - (double)vxcent) * x_pixel_scale;
-
-	if ( info.verbose & V_EVENT )
-	    printf ( "Button: delta position (lat/long) %.4f %.4f\n", dlat, dlong );
-
-	/* Make location of the mouse click be the current position */
-	if ( ! try_position ( info.long_deg + dlong, info.lat_deg - dlat ) )
-	    return;
-
 	for ( i=0; i<N_SERIES; i++ )
 	    info.series_info[i].content = 0;
 
@@ -551,6 +557,57 @@ move_xy ( int new_x, int new_y )
 	pixmap_expose ( 0, 0, vp_info.vx, vp_info.vy );
 }
 
+void
+move_xy ( int new_x, int new_y )
+{
+	int vxcent, vycent;
+	double dx, dy;
+
+	/* viewport center */
+	vxcent = vp_info.vx / 2;
+	vycent = vp_info.vy / 2;
+
+	if ( settings.verbose & V_EVENT )
+	    printf ( "Button: orig position (lat/long) %.4f %.4f\n",
+		info.lat_deg, info.long_deg );
+
+	/*
+	double x_pixel_scale, y_pixel_scale;
+	x_pixel_scale = info.series->maplet_long_deg / (double) info.series->xdim;
+	y_pixel_scale = info.series->maplet_lat_deg / (double) info.series->ydim;
+	*/
+
+	dx = (new_x - (double)vxcent) * info.series->x_pixel_scale;
+	dy = (new_y - (double)vycent) * info.series->y_pixel_scale;
+
+	if ( settings.verbose & V_EVENT )
+	    printf ( "Button: delta position (x/y) %.4f %.4f\n", dx, dy );
+
+	/* Make location of the mouse click be the current position */
+	if ( ! try_position ( dx, -dy ) )
+	    return;
+
+	full_redraw ();
+}
+
+void
+shift_xy ( double shift_x, double shift_y )
+{
+	double dx, dy;
+
+	dx = shift_x * info.series->x_pixel_scale;
+	dy = shift_y * info.series->y_pixel_scale;
+
+	if ( settings.verbose & V_EVENT )
+	    printf ( "Motion: delta position (x/y) %.4f %.4f\n", dx, dy );
+
+	/* Make location of the mouse click be the current position */
+	if ( ! try_position ( -dx, dy ) )
+	    return;
+
+	full_redraw ();
+}
+
 void
 move_map ( int dx, int dy )
 {
@@ -572,11 +629,138 @@ redraw_series ( void )
 {
 	if ( ! info.series->pixels )
 	    info.series->pixels = gdk_pixmap_new ( vp_info.da->window, vp_info.vx, vp_info.vy, -1 );
+
 	if ( ! info.series->content )
 	    pixmap_redraw ();
+
 	pixmap_expose ( 0, 0, vp_info.vx, vp_info.vy );
 }
 
+gint
+info_destroy_handler ( GtkWidget *w, GdkEvent *event, gpointer data )
+{
+	i_info.status = GONE;
+
+	return FALSE;
+}
+
+gint
+go_button_handler ( GtkWidget *w, GdkEvent *event, gpointer data )
+{
+	const gchar *sp;
+	double b_long, b_lat;
+
+	sp = gtk_entry_get_text ( GTK_ENTRY(i_info.e_long) );
+	if ( sp == '\0' )
+	    return;
+	b_long = parse_dms ( (char *) sp );
+	if ( b_long > 0.0 )
+	    b_long = -b_long;
+
+	sp = gtk_entry_get_text ( GTK_ENTRY(i_info.e_lat) );
+	if ( sp == '\0' )
+	    return;
+	b_lat = parse_dms ( (char *) sp );
+
+	if ( b_lat < 24.0 || b_lat > 50.0 )
+	    return;
+	if ( b_long < -125.0 || b_long > -66.0 )
+	    return;
+
+	printf ( "long: %.4f\n", b_long );
+	printf ( "lat: %.4f\n", b_lat );
+
+	set_position ( b_long, b_lat );
+
+	full_redraw ();
+
+	return FALSE;
+}
+
+/* XXX - It seems to me, that in this (and many other) places there is
+ * a half pixel error, we call the center of a 400 pixel screen, 200, but
+ * then get coordinates as 0-399
+ */
+void
+info_update ( void )
+{
+	char str[64];
+	double c_lat, c_long;
+
+	if ( i_info.status != UP )
+	    return;
+
+	c_long = info.long_deg + (mouse_info.x-vp_info.vxcent) * info.series->x_pixel_scale;
+	sprintf ( str, "%.5f", c_long );
+	gtk_label_set_text ( GTK_LABEL(i_info.l_long), str );
+
+	c_lat = info.lat_deg - (mouse_info.y-vp_info.vycent) * info.series->y_pixel_scale;
+	sprintf ( str, "%.5f", c_lat );
+	gtk_label_set_text ( GTK_LABEL(i_info.l_lat), str );
+}
+
+void
+info_window ( void )
+{
+	GtkWidget *w;
+	GtkWidget *vb;
+	GtkWidget *hb1;
+	GtkWidget *hb2;
+
+	if ( i_info.status == UP ) {
+	    /* printf ( "hiding\n" ); */
+	    gtk_widget_hide_all ( i_info.main );
+	    i_info.status = HIDDEN;
+	    return;
+	}
+
+	if ( i_info.status == HIDDEN ) {
+	    /* printf ( "unhiding\n" ); */
+	    gtk_widget_show_all ( i_info.main );
+	    i_info.status = UP;
+	    info_update ();
+	    return;
+	}
+
+	if ( i_info.status == GONE ) {
+	    /* printf ( "uping\n" ); */
+	    i_info.main = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
+	    vb = gtk_vbox_new ( FALSE, 0 );
+	    gtk_container_add ( GTK_CONTAINER(i_info.main), vb );
+
+	    hb1 = gtk_hbox_new ( FALSE, 0 );
+	    gtk_container_add ( GTK_CONTAINER(vb), hb1 );
+	    w = gtk_label_new ( "Long:" );
+	    gtk_box_pack_start ( GTK_BOX(hb1), w, TRUE, TRUE, 0 );
+	    i_info.l_long = gtk_label_new ( "-A-" );
+	    gtk_box_pack_start ( GTK_BOX(hb1), i_info.l_long, TRUE, TRUE, 0 );
+	    i_info.e_long = gtk_entry_new_with_max_length ( 16 );
+	    gtk_box_pack_start ( GTK_BOX(hb1), i_info.e_long, TRUE, TRUE, 0 );
+
+	    hb2 = gtk_hbox_new ( FALSE, 0 );
+	    gtk_container_add ( GTK_CONTAINER(vb), hb2 );
+	    w = gtk_label_new ( "Lat: " );
+	    gtk_box_pack_start ( GTK_BOX(hb2), w, TRUE, TRUE, 0 );
+	    i_info.l_lat = gtk_label_new ( "-B-" );
+	    gtk_box_pack_start ( GTK_BOX(hb2), i_info.l_lat, TRUE, TRUE, 0 );
+	    i_info.e_lat = gtk_entry_new_with_max_length ( 16 );
+	    gtk_box_pack_start ( GTK_BOX(hb2), i_info.e_lat, TRUE, TRUE, 0 );
+
+	    w = gtk_button_new_with_label ( "Go" );
+	    gtk_container_add ( GTK_CONTAINER(vb), w );
+	    g_signal_connect ( w, "clicked",
+			G_CALLBACK(go_button_handler), NULL );
+
+	    g_signal_connect ( i_info.main, "delete_event",
+			G_CALLBACK(info_destroy_handler), NULL );
+
+	    gtk_widget_show_all ( i_info.main );
+	    i_info.status = UP;
+
+	    info_update ();
+	}
+}
+
 #define KV_PAGE_UP	65365
 #define KV_PAGE_DOWN	65366
 
@@ -587,13 +771,41 @@ redraw_series ( void )
 
 #define KV_CTRL		65507
 
+#define KV_A		'a'
+#define KV_A_UC		'A'
+
+#define KV_I		'i'
+#define KV_I_UC		'I'
+
+#define KV_S		's'
+#define KV_S_UC		'S'
+
+/* We don't use these yet, but ... */
+#define KV_ESC		65307
+#define KV_TAB		65289
+
+void
+local_up_series ( void )
+{
+	up_series ();
+	info_update ();
+}
+
+void
+local_down_series ( void )
+{
+	down_series ();
+	info_update ();
+}
+
+
 /* Used to modify mouse actions */
 int ctrl_key_pressed = 0;
 
 gint
 keyboard_handler ( GtkWidget *wp, GdkEventKey *event, gpointer data )
 {
-	if ( info.verbose & V_EVENT ) {
+	if ( settings.verbose & V_EVENT ) {
 	    printf ( "Keyboard event %d %s",
 		event->keyval, gdk_keyval_name(event->keyval) );
 
@@ -621,9 +833,13 @@ keyboard_handler ( GtkWidget *wp, GdkEventKey *event, gpointer data )
 	    return TRUE;
 	
 	if ( event->keyval == KV_PAGE_UP )
-	    up_series ();
+	    local_up_series ();
 	else if ( event->keyval == KV_PAGE_DOWN )
-	    down_series ();
+	    local_down_series ();
+	else if ( event->keyval == KV_I || event->keyval == KV_I_UC )
+	    info_window ();
+	else if ( event->keyval == KV_S || event->keyval == KV_S_UC )
+	    snap ();
 	else if ( event->keyval == KV_LEFT )
 	    move_map ( -1, 0 );
 	else if ( event->keyval == KV_UP )
@@ -641,9 +857,13 @@ static int cursor_mode = 0;
 static void
 cursor_show ( int clean )
 {
-	if ( ! info.center_dot )
+	int size;
+
+	if ( ! settings.center_marker )
 	    return;
 
+	size = settings.marker_size;
+
 	if ( clean )
 	    cursor_mode = 1;
 	else
@@ -652,12 +872,12 @@ cursor_show ( int clean )
 	if ( cursor_mode ) {
 	    gdk_draw_line ( vp_info.da->window,
 		vp_info.da->style->white_gc,
-		vp_info.vxcent, vp_info.vycent-2,
-		vp_info.vxcent, vp_info.vycent+2 );
+		vp_info.vxcent, vp_info.vycent-size,
+		vp_info.vxcent, vp_info.vycent+size );
 	    gdk_draw_line ( vp_info.da->window,
 		vp_info.da->style->white_gc,
-		vp_info.vxcent-2, vp_info.vycent,
-		vp_info.vxcent+2, vp_info.vycent );
+		vp_info.vxcent-size, vp_info.vycent,
+		vp_info.vxcent+size, vp_info.vycent );
 	    gdk_draw_line ( vp_info.da->window,
 		vp_info.da->style->black_gc,
 		vp_info.vxcent, vp_info.vycent-1,
@@ -669,12 +889,12 @@ cursor_show ( int clean )
 	} else {
 	    gdk_draw_line ( vp_info.da->window,
 		vp_info.da->style->black_gc,
-		vp_info.vxcent, vp_info.vycent-2,
-		vp_info.vxcent, vp_info.vycent+2 );
+		vp_info.vxcent, vp_info.vycent-size,
+		vp_info.vxcent, vp_info.vycent+size );
 	    gdk_draw_line ( vp_info.da->window,
 		vp_info.da->style->black_gc,
-		vp_info.vxcent-2, vp_info.vycent,
-		vp_info.vxcent+2, vp_info.vycent );
+		vp_info.vxcent-size, vp_info.vycent,
+		vp_info.vxcent+size, vp_info.vycent );
 	}
 }
 
@@ -694,24 +914,30 @@ mouse_handler ( GtkWidget *wp, GdkEventButton *event, gpointer data )
 {
 	int button;
 
-	if ( info.verbose & V_EVENT )
+	if ( settings.verbose & V_EVENT )
 	    printf ( "Button event %d %.3f %.3f in (%d %d)\n",
 		event->button, event->x, event->y, vp_info.vx, vp_info.vy );
 
 	if ( event->button == 3 ) {
-	    if ( ctrl_key_pressed )
-	    	up_series ();
-	    else
-	    	down_series ();
-	    return TRUE;
+	    if ( settings.m3_action == M3_CENTER )
+		move_xy ( event->x, event->y );
+	    if ( settings.m3_action == M3_ZOOM ) {
+		if ( ctrl_key_pressed )
+		    local_up_series ();
+		else
+		    local_down_series ();
+	    }
 	}
 
 	if ( event->button == 2 ) {
+	    /* XXX */
 	    show_pos ();
-	    return TRUE;
 	}
 
-	move_xy ( event->x, event->y );
+	if ( event->button == 1 ) {
+	    if ( settings.m1_action == M1_CENTER )
+		move_xy ( event->x, event->y );
+	}
 
 	return TRUE;
 }
@@ -725,32 +951,85 @@ mouse_handler ( GtkWidget *wp, GdkEventButton *event, gpointer data )
 gint
 motion_handler ( GtkWidget *wp, GdkEventMotion *event, gpointer data )
 {
-	int button;
 	int x, y;
 	GdkModifierType state;
+	int dt;
+	double dx, dy;
 
-	/*
-	if ( info.verbose & V_EVENT )
-	    printf ( "Motion event %8x %.3f %.3f in (%d %d)\n",
-		event->state, event->x, event->y, vp_info.vx, vp_info.vy );
-	*/
+	if ( settings.verbose & V_EVENT )
+	    printf ( "Motion event  %u %8x %.3f %.3f in (%d %d)\n",
+		event->time, event->state, event->x, event->y, vp_info.vx, vp_info.vy );
+
+	/* Stash this (mostly to cleanly startup the info window)
+	 */
+	mouse_info.x = event->x;
+	mouse_info.y = event->y;
+	mouse_info.time = event->time;
+
+	/* plain old mouse moving around with no buttons down */
+	if ( event->state == 0 ) {
+	    info_update ();
+	    return;
+	}
 
 	/*
 	move_xy ( event->x, event->y );
 	*/
 
+	/* moving mouse with left button down - shift map */
+	if ( event->state & GDK_BUTTON1_MASK ) {
+	    dt = event->time - vp_info.mo_time;
+	    dx = event->x - vp_info.mo_x;
+	    dy = event->y - vp_info.mo_y;
+
+	    /*
+	    printf ( "dtxy = %d %.3f %.3f\n", dt, dx, dy );
+	    */
+	    if ( settings.m1_action == M1_GRAB && dt < 200 )
+		shift_xy ( dx, dy );
+	}
+
+	vp_info.mo_x = event->x;
+	vp_info.mo_y = event->y;
+	vp_info.mo_time = event->time;
+
 	/* It seems silly to call gdk_window_get_pointer since we get all
 	 * the same info anyway in the event structure, BUT this is not
 	 * quite true, notice the check on is_hint, This lets GDK know we
 	 * are done processing this event and are ready for another one,
 	 * and indeed without this call, we maybe get one event per second.
+	 * Note that this function returns integer (x,y), whereas the event
+	 * structure returns floating point (x,y).
 	 */
-	if ( event->is_hint )
+	if ( event->is_hint ) {
 	    gdk_window_get_pointer ( event->window, &x, &y, &state );
 
+	    /*
+	    event->x = x;
+	    event->y = y;
+	    */
+	    /*
+	    if ( x != event->x || y != event->y )
+		printf ( "Motion event2 %u %8x %d %d W(%x)\n",
+		    event->time, event->state, x, y, event->window );
+	    */
+	}
+
+
 	return TRUE;
 }
 
+gint
+scroll_handler ( GtkWidget *wp, GdkEventScroll *event, gpointer data )
+{
+	if ( event->direction == GDK_SCROLL_UP )
+	    local_up_series ();
+	else if ( event->direction == GDK_SCROLL_DOWN )
+	    local_down_series ();
+	else
+	    printf ( "Scroll event %d\n", event->direction );
+}
+
 /* Focus events are a funky business, but there is no way to
  * get the keyboard involved without handling them.
  */
@@ -758,64 +1037,114 @@ gint
 focus_handler ( GtkWidget *wp, GdkEventFocus *event, gpointer data )
 {
 	/* We get 1 when we go in, 0 when we go out */
-	if ( info.verbose & V_EVENT )
+	if ( settings.verbose & V_EVENT )
 	    printf ( "Focus event %d\n", event->in );
 }
 
 void
 synch_position ( void )
 {
-    	double m_lat, m_long;
+    	double x, y;
 
-	/*
-	int maplets;
-	maplets = tp->e_long / tp->maplet_long_deg;
-	tp->lat_offset = tp->e_long - maplets * tp->maplet_long_deg;
-	maplets = tp->s_lat / tp->maplet_lat_deg;
-	tp->long_offset = tp->s_lat - maplets * tp->maplet_lat_deg;
-	*/
+	if ( info.series->terra ) {
+	    ll_to_utm ( info.long_deg, info.lat_deg, &info.utm_zone, &info.utm_x, &info.utm_y );
+	    x = info.utm_x / ( 200.0 * info.series->x_pixel_scale );
+	    y = info.utm_y / ( 200.0 * info.series->y_pixel_scale );
 
-    	m_lat = (info.lat_deg - info.series->lat_offset) / info.series->maplet_lat_deg;
-    	m_long = - (info.long_deg - info.series->long_offset) / info.series->maplet_long_deg;
+	} else {
+	    /*
+	    int maplets;
+	    maplets = tp->e_long / tp->maplet_long_deg;
+	    tp->lat_offset = tp->e_long - maplets * tp->maplet_long_deg;
+	    maplets = tp->s_lat / tp->maplet_lat_deg;
+	    tp->long_offset = tp->s_lat - maplets * tp->maplet_lat_deg;
+	    */
+
+	    x = - (info.long_deg - info.series->long_offset) / info.series->maplet_long_deg;
+	    y =   (info.lat_deg - info.series->lat_offset) / info.series->maplet_lat_deg;
+	}
 
 	/* indices of the maplet we are in
 	 */
-    	info.long_maplet = m_long;
-    	info.lat_maplet = m_lat;
+    	info.maplet_x = x;
+    	info.maplet_y = y;
 
-	if ( info.verbose & V_BASIC ) {
+	if ( settings.verbose & V_BASIC ) {
 	    printf ( "Synch position: long/lat = %.3f %.3f\n", info.long_deg, info.lat_deg );
 	    printf ( "maplet indices of position: %d %d\n",
-		info.long_maplet, info.lat_maplet );
+		info.maplet_x, info.maplet_y );
 	}
 
 	/* fractional offset of our position in that maplet
 	 */
-	info.fy = 1.0 - (m_lat - info.lat_maplet);
-	info.fx = 1.0 - (m_long - info.long_maplet);
+	info.fx = 1.0 - (x - info.maplet_x);
+	info.fy = 1.0 - (y - info.maplet_y);
+}
+
+void
+synch_position_utm ( void )
+{
+	double x, y;
+
+	utm_to_ll ( info.utm_zone, info.utm_x, info.utm_y, &info.long_deg, &info.lat_deg );
+
+	x = info.utm_x / ( 200.0 * info.series->x_pixel_scale );
+	y = info.utm_y / ( 200.0 * info.series->y_pixel_scale );
+
+	/* indices of the maplet we are in
+	 */
+    	info.maplet_x = x;
+    	info.maplet_y = y;
+
+	if ( settings.verbose & V_BASIC ) {
+	    printf ( "Synch position: long/lat = %.3f %.3f\n", info.long_deg, info.lat_deg );
+	    printf ( "maplet indices of position: %d %d\n",
+		info.maplet_x, info.maplet_y );
+	}
+
+	/* fractional offset of our position in that maplet
+	 */
+	info.fx = 1.0 - (x - info.maplet_x);
+	info.fy = 1.0 - (y - info.maplet_y);
 }
 
 /* Used by mouse routine to check that a possible new
  * position still has map coverage.
  */
-int
-try_position ( double new_long, double new_lat )
+static int
+try_position ( double dx, double dy )
 {
-    	double orig_long, orig_lat;
+    	double save1, save2;
 
-	orig_long = info.long_deg;
-	orig_lat = info.lat_deg;
+	if ( info.series->terra ) {
+	    save1 = info.utm_x;
+	    save2 = info.utm_y;
 
-	info.long_deg = new_long;
-	info.lat_deg = new_lat;
-	synch_position ();
+	    info.utm_x -= dx;	/* XXX - note sign */
+	    info.utm_y += dy;
+	    synch_position_utm ();
+	} else {
+	    save1 = info.long_deg;
+	    save2 = info.lat_deg;
 
-	if ( load_maplet ( info.long_maplet, info.lat_maplet ) )
+	    info.long_deg += dx;
+	    info.lat_deg += dy;
+	    synch_position ();
+	}
+
+	if ( load_maplet ( info.maplet_x, info.maplet_y ) )
 	    return 1;
 
-	info.long_deg = orig_long;
-	info.lat_deg = orig_lat;
-	synch_position ();
+	/* Didn't like it, go back */
+	if ( info.series->terra ) {
+	    info.utm_x = save1;
+	    info.utm_y = save2;
+	    synch_position_utm ();
+	} else {
+	    info.long_deg = save1;
+	    info.lat_deg = save2;
+	    synch_position ();
+	}
 
 	return 0;
 }
@@ -846,14 +1175,18 @@ main ( int argc, char **argv )
 	GtkWidget *eb;
 	char *p;
 	char *file_name;
-	int view;
-	int start_series;
 
-#ifdef notdef
-	if ( ! temp_init() ) {
-	    printf ("Sorry, I can't find a place to put temporary files\n");
-	    return 1;
-	}
+	/* Built in places to look for maps, can be overridden
+	 * from the settings file.
+	 */
+#ifdef INITIAL_ARCHIVE
+	archive_add ( INITIAL_ARCHIVE );
+#else
+	archive_add ( "/u1/topo" );
+	archive_add ( "/u2/topo" );
+	archive_add ( "/mmt/topo" );
+	archive_add ( "/home/topo" );
+	archive_add ( "/topo" );
 #endif
 
 	/* Let gtk strip off any of its arguments first
@@ -863,43 +1196,43 @@ main ( int argc, char **argv )
 	argc--;
 	argv++;
 
-	info.verbose = INITIAL_VERBOSITY;
+	settings_init ();
+	places_init ();
+
+#ifdef notdef
+	/* Temporary hack XXX */
+	settings.verbose = V_EVENT;
+#endif
 
-	info.center_only = 0;
-	info.center_dot = 1;
 	info.series_info = series_info_buf;
-	info.show_maplets = 0;
-
-	view = INITIAL_VIEW;
-
-	start_series = INITIAL_SERIES;
+	info.center_only = 0;
 
+	/* XXX - really should dynamically generate version string at compile time */
 	while ( argc-- ) {
 	    p = *argv++;
-	    if ( strcmp ( p, "-v" ) == 0 )
-	    	info.verbose = 0xffff;
+	    if ( strcmp ( p, "-V" ) == 0 )
+	    	settings.verbose = 0xffff;
+	    if ( strcmp ( p, "-v" ) == 0 ) {
+	    	printf ( "gtopo version 0.9.10\n" );
+		return 0;
+	    }
 	    if ( strcmp ( p, "-c" ) == 0 )
 	    	info.center_only = 1;
 	    if ( strcmp ( p, "-d" ) == 0 )
-	    	info.center_dot = 0;
+	    	settings.center_marker = 0;
 	    if ( strcmp ( p, "-m" ) == 0 )
-	    	info.show_maplets = 1;
-	    if ( strcmp ( p, "-s" ) == 0 ) {
-		if ( argc < 1 )
-		    usage ();
-		argc--;
-		/* XXX - accept a number 1-5, pretty gross */
-		start_series = atol ( *argv++ ) - 1;
+	    	settings.show_maplets = 1;
+	    if ( strcmp ( p, "-h" ) == 0 ) {
+	    	http_test ();
+		return 0;
 	    }
-	    if ( strcmp ( p, "-g" ) == 0 ) {
-		/* won't the standard X geometry options work here?
-		 * (if I cooperate and do not
-		 * brute force resize and override ...
-		 */
-		if ( argc < 1 )
-		    usage ();
-		argc--;
-		view = atol ( *argv++ );
+	    if ( strcmp ( p, "-t" ) == 0 ) {
+	    	terra_test ();
+		return 0;
+	    }
+	    if ( strcmp ( p, "-x" ) == 0 ) {
+	    	xml_test ();
+		return 0;
 	    }
 	    if ( strcmp ( p, "-f" ) == 0 ) {
 		/* show a single tpq file */
@@ -932,7 +1265,7 @@ main ( int argc, char **argv )
 	    }
 	    printf ( "Displaying single file: %s\n", file_name );
 	} else {
-	    if ( ! archive_init ( topo_archives ) ) {
+	    if ( ! archive_init () ) {
 		printf ( "No topo archives found\n" );
 		return 1;
 	    }
@@ -940,15 +1273,8 @@ main ( int argc, char **argv )
 
 	if ( ! file_opt ) {
 
-	    /* not strictly needed, but set_series will access
-	     * these values.
-	     */
-	    info.long_deg = 0.0;
-	    info.lat_deg = 0.0;
-
-	    set_series ( start_series );
-
-	    set_position ( INITIAL_LONG, INITIAL_LAT );
+	    initial_series ( settings.starting_series );
+	    set_position ( settings.starting_long, settings.starting_lat );
 	}
 
 	/* --- set up the GTK stuff we need */
@@ -985,6 +1311,14 @@ main ( int argc, char **argv )
 	gtk_widget_add_events ( GTK_WIDGET(da), GDK_POINTER_MOTION_MASK );
 	gtk_widget_add_events ( GTK_WIDGET(da), GDK_POINTER_MOTION_HINT_MASK );
 
+	/* See if we can get scroll events (like the mouse wheel) */
+	g_signal_connect ( da, "scroll_event",
+			G_CALLBACK(scroll_handler), NULL );
+	/* Apparently not needed */
+	/*
+	gtk_widget_add_events ( GTK_WIDGET(da), GDK_SCROLL_MASK );
+	*/
+
 	/* Now, try to work the magic to get keyboard events */
 	GTK_WIDGET_SET_FLAGS ( da, GTK_CAN_FOCUS );
 	gtk_widget_add_events ( GTK_WIDGET(da), GDK_FOCUS_CHANGE_MASK );
@@ -1019,18 +1353,23 @@ main ( int argc, char **argv )
 	 */
 #endif
 
-	gtk_drawing_area_size ( GTK_DRAWING_AREA(da), view, view );
+	gtk_drawing_area_size ( GTK_DRAWING_AREA(da), settings.x_view, settings.y_view );
 
 	/*
+	#define MINIMUM_VIEW	100
 	gtk_drawing_area_size ( GTK_DRAWING_AREA(da), MINIMUM_VIEW, MINIMUM_VIEW );
-	gtk_widget_set_usize ( GTK_WIDGET(da), view, view );
-	gdk_window_resize ( da->window, view, view );
+	gtk_widget_set_usize ( GTK_WIDGET(da), settings.x_view, settings.y_view );
+	gdk_window_resize ( da->window, settings.x_view, settings.y_view );
 	*/
 
 	gtk_widget_show_all ( mw );
 
-	vp_info.vx = view;
-	vp_info.vy = view;
+	vp_info.vx = settings.x_view;
+	vp_info.vy = settings.y_view;
+
+	vp_info.mo_x = 0;
+	vp_info.mo_y = 0;
+	vp_info.mo_time = -10000;
 
 	gtk_main ();
 
