@@ -29,6 +29,7 @@
 #include "protos.h"
 
 extern struct topo_info info;
+extern struct settings settings;
 
 /* Using this is by FAR the way to go, and eliminates all the temp file
  * baloney
@@ -137,7 +138,7 @@ build_index_OLD ( struct tpq_info *tp, int fd, long *info )
 	    num_jpeg++;
 	}
 
-        index = (struct tpq_index_e *) malloc ( num_jpeg * sizeof(struct tpq_index_e) );
+        index = (struct tpq_index_e *) gmalloc ( num_jpeg * sizeof(struct tpq_index_e) );
         if ( ! index )
             error ("build_index, out of mem\n");
 
@@ -193,7 +194,7 @@ build_index ( struct tpq_info *tp, int fd )
 	 */
 	num_index = (offset - TPQ_HEADER_SIZE)/4;
 
-	proto_index = (struct tpq_index_e *) malloc ( num_index * sizeof(struct tpq_index_e) );
+	proto_index = (struct tpq_index_e *) gmalloc ( num_index * sizeof(struct tpq_index_e) );
 	if ( ! proto_index )
 	    error ("Build index: too many thingies %d!\n", num_index );
 
@@ -234,7 +235,7 @@ build_index ( struct tpq_info *tp, int fd )
 	/* Now copy to a smaller index buffer, and free the big one
 	 * we have been working in
 	 */
-        index = (struct tpq_index_e *) malloc ( num_jpeg * sizeof(struct tpq_index_e) );
+        index = (struct tpq_index_e *) gmalloc ( num_jpeg * sizeof(struct tpq_index_e) );
         if ( ! index )
 	    error ("Build index: too many maplets %d!\n", num_jpeg );
 
@@ -269,7 +270,7 @@ read_tpq_header ( struct tpq_info *tp, int fd )
 	if ( read( fd, &tpq_header, TPQ_HEADER_SIZE ) != TPQ_HEADER_SIZE )
 	    return 0;
 
-	if ( info.verbose & V_TPQ ) {
+	if ( settings.verbose & V_TPQ ) {
 	    printf ( "TPQ file for %s quadrangle: %s\n", tpq_header.state, tpq_header.quad_name );
 	    printf ( "TPQ file maplet counts long/lat: %d %d\n", tpq_header.maplet.nlong, tpq_header.maplet.nlat );
 	    printf ( "TPQ file long range: %.3f %3f\n", tpq_header.west_long, tpq_header.east_long );
@@ -391,7 +392,7 @@ read_tpq_header ( struct tpq_info *tp, int fd )
 	tp->maplet_long_deg = (tp->e_long - tp->w_long) / tp->long_count;
 	tp->maplet_lat_deg = (tp->n_lat - tp->s_lat) / tp->lat_count;
 
-	if ( info.verbose & V_TPQ ) {
+	if ( settings.verbose & V_TPQ ) {
 	    printf ( "TPQ file for %s quadrangle: %s\n", tp->state, tp->quad );
 	    printf ( "TPQ file maplet counts long/lat: %d %d\n", tp->long_count, tp->lat_count );
 	    printf ( "TPQ file long range: %.3f %3f\n", tp->w_long, tp->e_long );
@@ -455,10 +456,10 @@ tpq_new ( char *path )
         struct tpq_info *tp;
 	int fd;
 
-	if ( info.verbose & V_TPQ )
+	if ( settings.verbose & V_TPQ )
 	    printf ( "tpq_new: %s\n", path );
 
-        tp = (struct tpq_info *) malloc ( sizeof(struct tpq_info) );
+        tp = (struct tpq_info *) gmalloc ( sizeof(struct tpq_info) );
         if ( ! tp )
             error ("tpq_new, out of mem\n");
 
@@ -614,10 +615,10 @@ load_tpq_maplet ( struct maplet *mp )
 	if ( mp->tpq_index < 0 ) {
 	    x_index = tp->long_count / 2;
 	    y_index = tp->lat_count / 2;
-	    mp->sheet_index_long = tp->long_count - x_index - 1;
-	    mp->sheet_index_lat = tp->lat_count - y_index - 1;
-	    mp->world_index_long = tp->sheet_long + mp->sheet_index_long;
-	    mp->world_index_lat = tp->sheet_lat + mp->sheet_index_lat;
+	    mp->sheet_x = tp->long_count - x_index - 1;
+	    mp->sheet_y = tp->lat_count - y_index - 1;
+	    mp->world_x = tp->sheet_long + mp->sheet_x;
+	    mp->world_y = tp->sheet_lat + mp->sheet_y;
 	    mp->tpq_index = y_index * tp->long_count + x_index;
 	}
 
@@ -642,7 +643,7 @@ load_tpq_maplet ( struct maplet *mp )
 	    nw = size < BUFSIZE ? size : BUFSIZE;
 	    if ( read( fd, buf, nw ) != nw )
 		error ( "TPQ file read error %s %d %d\n", mp->tpq_path, off, size );
-	    gdk_pixbuf_loader_write ( loader, buf, nw, NULL );
+	    gdk_pixbuf_loader_write ( loader, (guchar *)buf, nw, NULL );
 	    size -= nw;
 	}
 
@@ -685,7 +686,7 @@ load_tpq_maplet ( struct maplet *mp )
 	remove ( tmpname );
 #endif
 
-	if ( ! mp->pixbuf && info.verbose & V_TPQ ) {
+	if ( ! mp->pixbuf && settings.verbose & V_TPQ ) {
 	    printf ("Cannot get pixbuf from %s (%d)\n", mp->tpq_path, mp->tpq_index );
 	    return 0;
 	}
